--[[Author:		Cyprias 
	License:	All Rights Reserved
	Contact:	Cyprias on Curse.com or WowAce.com

	Only Curse.com and Wowace.com have permission to host this addon.
	I have not given permission for My Sales to be used in any addon compilation or UI pack.]]
-- global lookup
local GetInboxNumItems_ = GetInboxNumItems
local GetInboxHeaderInfo_ = GetInboxHeaderInfo
local GetInboxInvoiceInfo_ = GetInboxInvoiceInfo
local date_ = date
local table_getn = table.getn
local table_insert = table.insert
local InterfaceOptionsFrame_OpenToCategory_ = InterfaceOptionsFrame_OpenToCategory
local GetGuildRosterInfo_ = GetGuildRosterInfo
local GetNumGuildMembers_ = GetNumGuildMembers
local table_remove = table.remove
local GetItemInfo_ = GetItemInfo
local GetItemQualityColor_ = GetItemQualityColor
local strsplit_ = strsplit

local folder, core = ...
MS = core --global table

-- Isolate the environment
local _G = getfenv(0)
setmetatable(MS, {__index = _G})
setfenv(1, MS)
-- local
title		= "My Sales"
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
titleFull	= title.." "..version

LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceSerializer-3.0", "AceHook-3.0", "AceEvent-3.0") 
local ScrollingTable = LibStub("ScrollingTable")
L = LibStub("AceLocale-3.0"):GetLocale(folder, true)

local regEvents = {
	"GUILD_ROSTER_UPDATE",
	"FRIENDLIST_UPDATE",
	"MAIL_CLOSED",
}
P	= {} --db.profile
fR	= {} --db.factionrealm
G	= {} --db.global
db	= {}
defaultSettings	= {
	factionrealm = {
		sales = {},
		PoI = {},
	},
	profile = {},
	global = {
		itemColour = {},
	}
}
guildMembers = {}
friendsList = {}
local copperMade = 0
local mailOpen = 0
local onlineColorTable = { ["r"] = 0.8, ["g"] = 0.8, ["b"] = 0.8, ["a"] = 1.0 } --gray buyer name


listFrame = false	--our main frame.
tableData = {}--data in the scrolling table.

function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("MS_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	self:RegisterChatCommand("ms", "MySlashProcessorFunc")
	
	
	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, titleFull)
end

function core:OnEnable()
	P = db.profile
	fR = db.factionrealm
	G = db.global
	
	self:Hook("TakeInboxMoney", true)

	for i, event in pairs(regEvents) do 
		self:RegisterEvent(event)
	end
	
	BuildPoIList()
	
	self:FRIENDLIST_UPDATE("FRIENDLIST_UPDATE")
end

function core:MAIL_CLOSED(event, ...)
	if P.salesMadeMessage == true and mailOpen > 0 then
		echo(L.receviedMail:format(mailOpen, CopperToCoins(copperMade)))
		mailOpen = 0
		copperMade = 0
	end
end


function core:OnDisable()
	
end


function core:GUILD_ROSTER_UPDATE(event, ...)
	guildMembers = {} --reset
	local numGuildMembers = GetNumGuildMembers_(true)
	local name
	for i=1, numGuildMembers do 
		name = GetGuildRosterInfo_(i);
		if name then
			guildMembers[name] = true
		end
	end
end


function core:FRIENDLIST_UPDATE(event, ...)
	friendsList = {} --reset
	local numberOfFriends = GetNumFriends()
	local name
	for i=1, numberOfFriends do 
		 name = GetFriendInfo(i)
		 if name then
			friendsList[name] = true
		end
	end
end


function GetBuyerColour(data, cols, realrow, column, table)
	local buyerName = data[realrow].cols[column].value

	if guildMembers[buyerName] then
		return P.guildColour
	elseif friendsList[buyerName] then
		return P.friendColour
	elseif fR.PoI[buyerName] then
		return P.PoIColour
	end
	return onlineColorTable
end


function GetItemColour(data, cols, realrow, column, table)
	local itemName = data[realrow].cols[column].value
	if G.itemColour[itemName] then
		local r,g,b = strsplit_(",", G.itemColour[itemName])
		return {r=r,g=g,b=b,a=1}
	end
	
	local _, _, iRarity = GetItemInfo_(itemName);
	local r, g, b, hex = GetItemQualityColor_(iRarity or 0)
	r = Round(r,2)
	g = Round(g,2)
	b = Round(b,2)

	if iRarity then
		G.itemColour[itemName] = ("%s,%s,%s"):format(r,g,b)
		return {r=r,g=g,b=b,a=1}
	end
	
	return {r=r,g=g,b=b,a=1}
end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /da function brings up the UI options.	--
----------------------------------------------
--~ 	InterfaceOptionsFrame_OpenToCategory_(profitUI)--Expands the menu.
	InterfaceOptionsFrame_OpenToCategory_(coreOpts)
end

function core:TakeInboxMoney(...)
	local mailIndex  = ...

	if GetInboxNumItems_() >= mailIndex then
		local _, _, sender, subject, money = GetInboxHeaderInfo_(mailIndex);
		local _, itemName, playerName = GetInboxInvoiceInfo_(mailIndex);


		if sender:find(L["Auction House"]) and subject:find(L["Auction successful:"]) then --locale
			if not playerName or playerName == "" then
				Debug("TakeInboxMoney", "No buyer", sender, subject, money, itemName, playerName)
			end
			local seral = self:Serialize(itemName, money, playerName, date_("%c"))
--~ 			Debug("TakeInboxMoney", "seral", seral)
			table_insert(fR.sales, #fR.sales+1, seral)
			
			mailOpen = mailOpen + 1
			copperMade = copperMade + money
		end
	end
end

function RefreshTable()--		/run MS.RefreshTable()
	if not listFrame then
		local AceGUI = LibStub("AceGUI-3.0")
		listFrame = listFrame or AceGUI:Create("Frame")
		local window = listFrame.frame;

		window:SetHeight(460);
		listFrame:SetCallback("OnClose",function(widget) AceGUI:Release(widget) end)
		listFrame:SetTitle(title)
		listFrame:SetLayout("Flow")

		local cols = {
			{ name= L["Item"], width = 200, defaultsort = "dsc", },
			{ name= L["Money"], width = 80, defaultsort = "dsc", 
				comparesort = function (self, rowa, rowb, sortbycol)
					if tableData[rowa] and tableData[rowb] then
						if self.cols[sortbycol].sort == "dsc" then
							return tableData[rowa].cols[sortbycol].args[1] < tableData[rowb].cols[sortbycol].args[1]
						else
							return tableData[rowa].cols[sortbycol].args[1] > tableData[rowb].cols[sortbycol].args[1]
						end
					end
				end,
			},
			{ name= L["Buyer"], width = 80, defaultsort = "dsc", },
			{ name= L["When"], width = 180, defaultsort = "dsc",
				comparesort = function (self, rowa, rowb, sortbycol)
					if tableData[rowa] and tableData[rowb] then
						if self.cols[sortbycol].sort == "dsc" then
							return tableData[rowa].cols[sortbycol].args[1] < tableData[rowb].cols[sortbycol].args[1]
						else
							return tableData[rowa].cols[sortbycol].args[1] > tableData[rowb].cols[sortbycol].args[1]
						end
					end
				end,
			},
		};
		
		local width = 0
		for i, data in pairs(cols) do 
			width = width + data.width
		end
		window:SetWidth(width);
		
		local st = ScrollingTable:CreateST(cols, 20, 16, nil, window)
		st.frame:SetPoint("BOTTOMLEFT",20,80)
		st.frame:SetPoint("TOP", window, 0, -45)
		st.frame:SetPoint("RIGHT", window, -20,0)
		listFrame.st = st
	end
	tableData = {}
	
	local totalCopper = 0
	local success, who, data
	local secSinceWhen
	local allData = {}
	for i=1, table_getn(fR.sales) do 
		success, item, money, buyer, when = core:Deserialize(fR.sales[i])
		if success then
			secSinceWhen = SecondsSinceDateString(when)
			if P.filterWhen == 0 or secSinceWhen <= P.filterWhen then
				if P.showBuyers == 1 or (P.showBuyers == 2 and guildMembers[buyer]) or (P.showBuyers == 3 and friendsList[buyer]) or( P.showBuyers == 4 and fR.PoI[buyer]) then
			
			
					totalCopper = totalCopper + money
					table.insert(tableData, {
						cols = {
							{
								value = item, 
								color = GetItemColour
							}, 
							{
								value = function (copper)
									return CopperToCoins(copper)
								end,
								args = {money},
							},
							{
								value = buyer, 
								color=GetBuyerColour
							},
							{
								value = function (secondsSince, when)
									if P.relativeWhen == true then
										return SecondsToString(secondsSince).." "..L["ago"]
									else
										return when
									end
								end,
								args = {secSinceWhen, when},
							},
						}
					})
				end
			end
		end
	end
	
	listFrame:SetStatusText(L["Total"]..": "..CopperToCoins(totalCopper))
	
	listFrame.st:SetData(tableData)
end--	/run MS.RefreshTable()

